@@ -18,9 +18,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 #ifndef __CLUTTER_H__
@@ -30,7 +30,10 @@
 
 #include "clutter-deprecated.h"
 
+#include "clutter-action.h"
 #include "clutter-actor.h"
+#include "clutter-actor-meta.h"
+#include "clutter-align-constraint.h"
 #include "clutter-alpha.h"
 #include "clutter-animatable.h"
 #include "clutter-animation.h"
@@ -43,16 +46,25 @@
 #include "clutter-behaviour-path.h"
 #include "clutter-behaviour-rotate.h"
 #include "clutter-behaviour-scale.h"
+#include "clutter-bind-constraint.h"
 #include "clutter-binding-pool.h"
 #include "clutter-bin-layout.h"
+#include "clutter-blur-effect.h"
 #include "clutter-box.h"
 #include "clutter-box-layout.h"
 #include "clutter-cairo-texture.h"
 #include "clutter-child-meta.h"
+#include "clutter-click-action.h"
 #include "clutter-clone.h"
 #include "clutter-color.h"
+#include "clutter-colorize-effect.h"
+#include "clutter-constraint.h"
 #include "clutter-container.h"
+#include "clutter-deform-effect.h"
+#include "clutter-desaturate-effect.h"
 #include "clutter-device-manager.h"
+#include "clutter-drag-action.h"
+#include "clutter-effect.h"
 #include "clutter-event.h"
 #include "clutter-feature.h"
 #include "clutter-fixed-layout.h"
@@ -68,16 +80,21 @@
 #include "clutter-main.h"
 #include "clutter-media.h"
 #include "clutter-model.h"
+#include "clutter-offscreen-effect.h"
+#include "clutter-page-turn-effect.h"
 #include "clutter-path.h"
 #include "clutter-rectangle.h"
 #include "clutter-score.h"
 #include "clutter-scriptable.h"
 #include "clutter-script.h"
+#include "clutter-settings.h"
 #include "clutter-shader.h"
+#include "clutter-shader-effect.h"
 #include "clutter-shader-types.h"
 #include "clutter-stage.h"
 #include "clutter-stage-manager.h"
 #include "clutter-stage-window.h"
+#include "clutter-state.h"
 #include "clutter-texture.h"
 #include "clutter-text.h"
 #include "clutter-timeline.h"
