@@ -36,7 +36,7 @@
 #include "cogl-bitmap.h"
 #include "cogl-bitmap-private.h"
 #include "cogl-buffer-private.h"
-#include "cogl-pixel-buffer-private.h"
+#include "cogl-pixel-array-private.h"
 #include "cogl-texture-private.h"
 #include "cogl-texture-driver.h"
 #include "cogl-texture-2d-sliced-private.h"
@@ -59,18 +59,31 @@
  * abstract class manually.
  */
 
+void
+_cogl_texture_register_texture_type (GQuark type)
+{
+  _COGL_GET_CONTEXT (ctxt, NO_RETVAL);
+
+  ctxt->texture_types = g_slist_prepend (ctxt->texture_types,
+                                         GINT_TO_POINTER (type));
+}
+
 gboolean
 cogl_is_texture (CoglHandle handle)
 {
   CoglHandleObject *obj = (CoglHandleObject *)handle;
+  GSList *l;
+
+  _COGL_GET_CONTEXT (ctxt, FALSE);
 
   if (handle == COGL_INVALID_HANDLE)
     return FALSE;
 
-  return (obj->klass->type == _cogl_handle_texture_2d_get_type () ||
-          obj->klass->type == _cogl_handle_atlas_texture_get_type () ||
-          obj->klass->type == _cogl_handle_texture_2d_sliced_get_type () ||
-          obj->klass->type == _cogl_handle_sub_texture_get_type ());
+  for (l = ctxt->texture_types; l; l = l->next)
+    if (GPOINTER_TO_INT (l->data) == obj->klass->type)
+      return TRUE;
+
+  return FALSE;
 }
 
 CoglHandle
@@ -102,12 +115,19 @@ cogl_texture_unref (CoglHandle handle)
   cogl_handle_unref (handle);
 }
 
+void
+_cogl_texture_free (CoglTexture *texture)
+{
+  g_free (texture);
+}
+
 static gboolean
 _cogl_texture_needs_premult_conversion (CoglPixelFormat src_format,
                                         CoglPixelFormat dst_format)
 {
-  return ((src_format & COGL_A_BIT) &&
+  return ((src_format & dst_format & COGL_A_BIT) &&
           src_format != COGL_PIXEL_FORMAT_A_8 &&
+          dst_format != COGL_PIXEL_FORMAT_A_8 &&
           (src_format & COGL_PREMULT_BIT) !=
           (dst_format & COGL_PREMULT_BIT));
 }
@@ -244,12 +264,17 @@ _cogl_texture_prep_gl_alignment_for_pixels_download (int pixels_rowstride)
 
 /* FIXME: wrap modes should be set on materials not textures */
 void
-_cogl_texture_set_wrap_mode_parameter (CoglHandle handle,
-                                       GLenum wrap_mode)
+_cogl_texture_set_wrap_mode_parameters (CoglHandle handle,
+                                        GLenum wrap_mode_s,
+                                        GLenum wrap_mode_t,
+                                        GLenum wrap_mode_r)
 {
   CoglTexture *tex = COGL_TEXTURE (handle);
 
-  tex->vtable->set_wrap_mode_parameter (tex, wrap_mode);
+  tex->vtable->set_wrap_mode_parameters (tex,
+                                         wrap_mode_s,
+                                         wrap_mode_t,
+                                         wrap_mode_r);
 }
 
 /* This is like CoglSpanIter except it deals with floats and it
@@ -466,6 +491,8 @@ cogl_texture_new_from_foreign (GLuint           gl_handle,
 			       GLuint           y_pot_waste,
 			       CoglPixelFormat  format)
 {
+  /* FIXME: only create a sliced texture if x or y waste was specified
+   */
   return _cogl_texture_2d_sliced_new_from_foreign (gl_handle,
                                                    gl_target,
                                                    width,
@@ -475,6 +502,21 @@ cogl_texture_new_from_foreign (GLuint           gl_handle,
                                                    format);
 }
 
+gboolean
+_cogl_texture_is_foreign (CoglHandle handle)
+{
+  CoglTexture *tex;
+
+  g_return_val_if_fail (cogl_is_texture (handle), FALSE);
+
+  tex = COGL_TEXTURE (handle);
+
+  if (tex->vtable->is_foreign)
+    return tex->vtable->is_foreign (tex);
+  else
+    return FALSE;
+}
+
 CoglHandle
 cogl_texture_new_from_sub_texture (CoglHandle full_texture,
                                    int        sub_x,
@@ -498,7 +540,7 @@ cogl_texture_new_from_buffer_EXP (CoglHandle          buffer,
 {
   CoglHandle texture;
   CoglBuffer *cogl_buffer;
-  CoglPixelBuffer *pixel_buffer;
+  CoglPixelArray *pixel_array;
 
   g_return_val_if_fail (cogl_is_buffer (buffer), COGL_INVALID_HANDLE);
 
@@ -506,23 +548,23 @@ cogl_texture_new_from_buffer_EXP (CoglHandle          buffer,
     return COGL_INVALID_HANDLE;
 
   cogl_buffer = COGL_BUFFER (buffer);
-  pixel_buffer = COGL_PIXEL_BUFFER (buffer);
+  pixel_array = COGL_PIXEL_ARRAY (buffer);
 
   /* Rowstride from CoglBuffer or even width * bpp if not given */
   if (rowstride == 0)
-    rowstride = pixel_buffer->stride;
+    rowstride = pixel_array->stride;
   if (rowstride == 0)
     rowstride = width * _cogl_get_format_bpp (format);
 
   /* use the CoglBuffer height and width as last resort */
   if (width == 0)
-    width = pixel_buffer->width;
+    width = pixel_array->width;
   if (height == 0)
-    height = pixel_buffer->height;
+    height = pixel_array->height;
   if (width == 0 || height == 0)
     {
       /* no width or height specified, neither at creation time (because the
-       * buffer was created by cogl_pixel_buffer_new()) nor when calling this
+       * array was created by cogl_pixel_array_new()) nor when calling this
        * function */
       return COGL_INVALID_HANDLE;
     }
@@ -539,9 +581,10 @@ cogl_texture_new_from_buffer_EXP (CoglHandle          buffer,
       bitmap.format = format;
       bitmap.rowstride = rowstride;
 
-      _cogl_buffer_bind (cogl_buffer, GL_PIXEL_UNPACK_BUFFER);
+      _cogl_buffer_bind (cogl_buffer,
+                         COGL_BUFFER_BIND_TARGET_PIXEL_UNPACK);
       texture =  cogl_texture_new_from_bitmap (&bitmap, flags, internal_format);
-      _cogl_buffer_bind (NULL, GL_PIXEL_UNPACK_BUFFER);
+      _cogl_buffer_unbind (cogl_buffer);
     }
   else
 #endif
@@ -744,7 +787,7 @@ _cogl_texture_set_filters (CoglHandle handle,
 }
 
 void
-_cogl_texture_ensure_mipmaps (CoglHandle handle)
+_cogl_texture_pre_paint (CoglHandle handle, CoglTexturePrePaintFlags flags)
 {
   CoglTexture *tex;
 
@@ -753,7 +796,7 @@ _cogl_texture_ensure_mipmaps (CoglHandle handle)
 
   tex = COGL_TEXTURE (handle);
 
-  tex->vtable->ensure_mipmaps (tex);
+  tex->vtable->pre_paint (tex, flags);
 }
 
 void
@@ -769,6 +812,51 @@ _cogl_texture_ensure_non_quad_rendering (CoglHandle handle)
   tex->vtable->ensure_non_quad_rendering (tex);
 }
 
+gboolean
+_cogl_texture_set_region_from_bitmap (CoglHandle    handle,
+                                      int           src_x,
+                                      int           src_y,
+                                      int           dst_x,
+                                      int           dst_y,
+                                      unsigned int  dst_width,
+                                      unsigned int  dst_height,
+                                      CoglBitmap   *bmp)
+{
+  CoglTexture *tex = COGL_TEXTURE (handle);
+  CoglBitmap   tmp_bmp;
+  gboolean     tmp_bmp_owner = FALSE;
+  GLenum       closest_gl_format;
+  GLenum       closest_gl_type;
+  gboolean     ret;
+
+  /* Shortcut out early if the image is empty */
+  if (dst_width == 0 || dst_height == 0)
+    return TRUE;
+
+  /* Prepare the bitmap so that it will do the premultiplication
+     conversion */
+  _cogl_texture_prepare_for_upload (bmp,
+                                    cogl_texture_get_format (handle),
+                                    NULL,
+                                    &tmp_bmp,
+                                    &tmp_bmp_owner,
+                                    NULL,
+                                    &closest_gl_format,
+                                    &closest_gl_type);
+
+  ret = tex->vtable->set_region (handle,
+                                 src_x, src_y,
+                                 dst_x, dst_y,
+                                 dst_width, dst_height,
+                                 &tmp_bmp);
+
+  /* Free data if owner */
+  if (tmp_bmp_owner)
+    g_free (tmp_bmp.data);
+
+  return ret;
+}
+
 gboolean
 cogl_texture_set_region (CoglHandle       handle,
 			 int              src_x,
@@ -783,21 +871,28 @@ cogl_texture_set_region (CoglHandle       handle,
 			 unsigned int     rowstride,
 			 const guint8    *data)
 {
-  CoglTexture *tex;
+  int              bpp;
+  CoglBitmap       source_bmp;
 
-  if (!cogl_is_texture (handle))
+  /* Check for valid format */
+  if (format == COGL_PIXEL_FORMAT_ANY)
     return FALSE;
 
-  tex = COGL_TEXTURE (handle);
+  /* Init source bitmap */
+  source_bmp.width = width;
+  source_bmp.height = height;
+  source_bmp.format = format;
+  source_bmp.data = (guint8 *) data;
 
-  return tex->vtable->set_region (tex,
-                                  src_x, src_y,
-                                  dst_x, dst_y,
-                                  dst_width, dst_height,
-                                  width, height,
-                                  format,
-                                  rowstride,
-                                  data);
+  /* Rowstride from width if none specified */
+  bpp = _cogl_get_format_bpp (format);
+  source_bmp.rowstride = (rowstride == 0) ? width * bpp : rowstride;
+
+  return _cogl_texture_set_region_from_bitmap (handle,
+                                               src_x, src_y,
+                                               dst_x, dst_y,
+                                               dst_width, dst_height,
+                                               &source_bmp);
 }
 
 /* Reads back the contents of a texture by rendering it to the framebuffer
@@ -875,13 +970,12 @@ do_texture_draw_and_read (CoglHandle   handle,
           rect_bmp.data = g_malloc (rect_bmp.rowstride *
                                     rect_bmp.height);
 
-          _cogl_texture_driver_prep_gl_for_pixels_download (rect_bmp.rowstride,
-                                                            bpp);
-          GE( glReadPixels (viewport[0], viewport[1],
+          cogl_read_pixels (viewport[0], viewport[1],
                             rect_bmp.width,
                             rect_bmp.height,
-                            GL_RGBA, GL_UNSIGNED_BYTE,
-                            rect_bmp.data) );
+                            COGL_READ_PIXELS_COLOR_BUFFER,
+                            COGL_PIXEL_FORMAT_RGBA_8888_PRE,
+                            rect_bmp.data);
 
           /* Copy to target bitmap */
           _cogl_bitmap_copy_subregion (&rect_bmp,
@@ -910,7 +1004,7 @@ _cogl_texture_draw_and_read (CoglHandle   handle,
                              GLuint       target_gl_type)
 {
   int        bpp;
-  CoglHandle framebuffer;
+  CoglFramebuffer *framebuffer;
   int        viewport[4];
   CoglBitmap alpha_bmp;
   CoglHandle prev_source;
@@ -928,8 +1022,8 @@ _cogl_texture_draw_and_read (CoglHandle   handle,
       viewport[2] <= 0 || viewport[3] <= 0)
     return FALSE;
 
-  /* Setup orthographic projection into current viewport (0,0 in bottom-left
-   * corner to draw the texture upside-down so we match the way glReadPixels
+  /* Setup orthographic projection into current viewport (0,0 in top-left
+   * corner to draw the texture upside-down so we match the way cogl_read_pixels
    * works)
    */
 
@@ -938,7 +1032,7 @@ _cogl_texture_draw_and_read (CoglHandle   handle,
   _cogl_matrix_stack_load_identity (projection_stack);
   _cogl_matrix_stack_ortho (projection_stack,
                             0, (float)(viewport[2]),
-                            0, (float)(viewport[3]),
+                            (float)(viewport[3]), 0,
                             (float)(0),
                             (float)(100));
 
@@ -966,6 +1060,10 @@ _cogl_texture_draw_and_read (CoglHandle   handle,
                                    "RGBA = REPLACE (TEXTURE)",
                                    NULL);
 
+  cogl_material_set_layer_filters (ctx->texture_download_material, 0,
+                                   COGL_MATERIAL_FILTER_NEAREST,
+                                   COGL_MATERIAL_FILTER_NEAREST);
+
   do_texture_draw_and_read (handle, target_bmp, viewport);
 
   /* Check whether texture has alpha and framebuffer not */
@@ -1041,13 +1139,105 @@ cogl_texture_get_data (CoglHandle       handle,
 		       unsigned int     rowstride,
 		       guint8          *data)
 {
-  CoglTexture *tex;
+  CoglTexture     *tex;
+  int              bpp;
+  int              byte_size;
+  CoglPixelFormat  closest_format;
+  int              closest_bpp;
+  GLenum           closest_gl_format;
+  GLenum           closest_gl_type;
+  CoglBitmap       target_bmp;
+  CoglBitmap       new_bmp;
+  gboolean         success;
+  guint8          *src;
+  guint8          *dst;
+  int              y;
+  int              tex_width;
+  int              tex_height;
 
   if (!cogl_is_texture (handle))
     return FALSE;
 
   tex = COGL_TEXTURE (handle);
 
-  return tex->vtable->get_data (handle, format, rowstride, data);
-}
+  /* Default to internal format if none specified */
+  if (format == COGL_PIXEL_FORMAT_ANY)
+    format = cogl_texture_get_format (handle);
 
+  tex_width = cogl_texture_get_width (handle);
+  tex_height = cogl_texture_get_height (handle);
+
+  /* Rowstride from texture width if none specified */
+  bpp = _cogl_get_format_bpp (format);
+  if (rowstride == 0)
+    rowstride = tex_width * bpp;
+
+  /* Return byte size if only that requested */
+  byte_size = tex_height * rowstride;
+  if (data == NULL)
+    return byte_size;
+
+  closest_format =
+    _cogl_texture_driver_find_best_gl_get_data_format (format,
+                                                       &closest_gl_format,
+                                                       &closest_gl_type);
+  closest_bpp = _cogl_get_format_bpp (closest_format);
+
+  target_bmp.width = tex_width;
+  target_bmp.height = tex_height;
+
+  /* Is the requested format supported? */
+  if (closest_format == format)
+    {
+      /* Target user data directly */
+      target_bmp.format = format;
+      target_bmp.rowstride = rowstride;
+      target_bmp.data = data;
+    }
+  else
+    {
+      /* Target intermediate buffer */
+      target_bmp.format = closest_format;
+      target_bmp.rowstride = target_bmp.width * closest_bpp;
+      target_bmp.data = g_malloc (target_bmp.height * target_bmp.rowstride);
+    }
+
+  if (!tex->vtable->get_data (tex,
+                              target_bmp.format,
+                              target_bmp.rowstride,
+                              target_bmp.data))
+    /* XXX: In some cases _cogl_texture_2d_download_from_gl may fail
+     * to read back the texture data; such as for GLES which doesn't
+     * support glGetTexImage, so here we fallback to drawing the
+     * texture and reading the pixels from the framebuffer. */
+    _cogl_texture_draw_and_read (tex, &target_bmp,
+                                 closest_gl_format,
+                                 closest_gl_type);
+
+  /* Was intermediate used? */
+  if (closest_format != format)
+    {
+      /* Convert to requested format */
+      success = _cogl_bitmap_convert_format_and_premult (&target_bmp,
+                                                         &new_bmp,
+                                                         format);
+
+      /* Free intermediate data and return if failed */
+      g_free (target_bmp.data);
+      if (!success)
+        return 0;
+
+      /* Copy to user buffer */
+      for (y = 0; y < new_bmp.height; ++y)
+        {
+          src = new_bmp.data + y * new_bmp.rowstride;
+          dst = data + y * rowstride;
+          memcpy (dst, src, new_bmp.width * bpp);
+        }
+
+      /* Free converted data */
+      g_free (new_bmp.data);
+    }
+
+  return byte_size;
+}
