@@ -37,11 +37,11 @@
 
 #include "cogl-shader-private.h"
 #include "cogl-program.h"
-#include "cogl-journal-private.h"
 
 static void _cogl_program_free (CoglProgram *program);
 
 COGL_HANDLE_DEFINE (Program, program);
+COGL_OBJECT_DEFINE_DEPRECATED_REF_COUNTING (program);
 
 static void
 _cogl_program_free (CoglProgram *program)
@@ -66,8 +66,6 @@ _cogl_program_free (CoglProgram *program)
   for (i = 0; i < COGL_GLES2_NUM_CUSTOM_UNIFORMS; i++)
     if (program->custom_uniform_names[i])
       g_free (program->custom_uniform_names[i]);
-
-  g_slice_free (CoglProgram, program);
 }
 
 CoglHandle
@@ -118,16 +116,22 @@ cogl_program_use (CoglHandle handle)
 {
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  if (handle != COGL_INVALID_HANDLE && !cogl_is_program (handle))
-    return;
+  g_return_if_fail (handle == COGL_INVALID_HANDLE ||
+                    cogl_is_program (handle));
 
-  /* The Cogl journal doesn't currently cope with the use of
-   * shaders so we have to flush all priitives whenever the
-   * current shader changes... */
-  _cogl_journal_flush ();
+  if (ctx->current_program == 0 && handle != 0)
+    ctx->legacy_state_set++;
+  else if (handle == 0 && ctx->current_program != 0)
+    ctx->legacy_state_set--;
 
   ctx->drv.gles2.settings.user_program = handle;
   ctx->drv.gles2.settings_dirty = TRUE;
+
+  if (handle != COGL_INVALID_HANDLE)
+    cogl_handle_ref (handle);
+  if (ctx->current_program != COGL_INVALID_HANDLE)
+    cogl_handle_unref (ctx->current_program);
+  ctx->current_program = handle;
 }
 
 int
