@@ -15,9 +15,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 #ifndef __CLUTTER_STAGE_OSX_H__
 #define __CLUTTER_STAGE_OSX_H__
@@ -50,7 +50,7 @@ typedef struct _ClutterStageOSXClass ClutterStageOSXClass;
 
 struct _ClutterStageOSX
 {
-  ClutterActor parent;
+  ClutterGroup parent;
 
   ClutterBackend *backend;
   ClutterStage   *wrapper;
@@ -69,13 +69,13 @@ struct _ClutterStageOSX
 
 struct _ClutterStageOSXClass
 {
-  ClutterActorClass parent_class;
+  ClutterGroupClass parent_class;
 };
 
 GType           clutter_stage_osx_get_type    (void) G_GNUC_CONST;
 
-ClutterActor*   clutter_stage_osx_new (ClutterBackend *backend,
-                                       ClutterStage   *wrapper);
+ClutterStageWindow*   clutter_stage_osx_new (ClutterBackend *backend,
+                                             ClutterStage   *wrapper);
 
 G_END_DECLS
 
